@@ -89,9 +89,12 @@ qpwgraph_canvas::qpwgraph_canvas ( QWidget *parent )
 	QGraphicsView::setResizeAnchor(QGraphicsView::NoAnchor);
 	QGraphicsView::setDragMode(QGraphicsView::NoDrag);
 
-	const QPalette& pal = QGraphicsView::palette();
-	if (pal.base().color().value() > 192)
-		QGraphicsView::setBackgroundRole(QPalette::Mid);
+	QPalette pal = QGraphicsView::palette();
+	const QPalette::ColorRole role = QPalette::Window;
+	const QColor& color = pal.color(role);
+	pal.setColor(role, color.darker(120));
+	QGraphicsView::setPalette(pal);
+	QGraphicsView::setBackgroundRole(role);
 
 	m_editor = new QLineEdit(this);
 	m_editor->setFrame(false);
@@ -592,13 +595,20 @@ void qpwgraph_canvas::emitDisconnected (
 }
 
 
-// Rename notifiers.
+// Rename notifier.
 void qpwgraph_canvas::emitRenamed ( qpwgraph_item *item, const QString& name )
 {
 	emit renamed(item, name);
 }
 
 
+// Other generic notifier.
+void qpwgraph_canvas::emitChanged (void)
+{
+	emit changed();
+}
+
+
 // Item finder (internal).
 qpwgraph_item *qpwgraph_canvas::itemAt ( const QPointF& pos ) const
 {
@@ -922,6 +932,7 @@ void qpwgraph_canvas::mouseReleaseEvent ( QMouseEvent *event )
 			}
 			m_commands->push(
 				new qpwgraph_move_command(this, nodes, m_pos1, pos));
+			++nchanged;
 		}
 		// Close rubber-band lasso...
 		if (m_rubberband) {
@@ -1395,6 +1406,11 @@ bool qpwgraph_canvas::restorePort ( qpwgraph_port *port )
 		return false;
 
 	port->setPortTitle(port_title);
+
+	qpwgraph_node *node = port->portNode();
+	if (node)
+		node->updatePath();
+
 	return true;
 }
 
