@@ -31,6 +31,7 @@
 
 #include "qpwgraph_patchbay.h"
 #include "qpwgraph_systray.h"
+#include "qpwgraph_thumb.h"
 
 #include <pipewire/pipewire.h>
 
@@ -41,6 +42,7 @@
 #include <QFileDialog>
 
 #include <QMessageBox>
+#include <QCheckBox>
 
 #include <QHBoxLayout>
 #include <QToolButton>
@@ -126,6 +128,9 @@ qpwgraph_form::qpwgraph_form (
 	m_systray = nullptr;
 	m_systray_closed = false;
 
+	m_thumb = nullptr;
+	m_thumb_update = 0;
+
 	QUndoStack *commands = m_ui.graphCanvas->commands();
 
 	QAction *undo_action = commands->createUndoAction(this, tr("&Undo"));
@@ -226,7 +231,7 @@ qpwgraph_form::qpwgraph_form (
 
 	QObject::connect(m_ui.graphCanvas,
 		SIGNAL(changed()),
-		SLOT(stabilize()));
+		SLOT(changed()));
 
 	// Some actions surely need those
 	// shortcuts firmly attached...
@@ -322,6 +327,36 @@ qpwgraph_form::qpwgraph_form (
 		SIGNAL(triggered(bool)),
 		SLOT(viewPatchbayToolbar(bool)));
 
+	m_thumb_mode = new QActionGroup(this);
+	m_thumb_mode->setExclusive(true);
+	m_thumb_mode->addAction(m_ui.viewThumbviewTopLeftAction);
+	m_thumb_mode->addAction(m_ui.viewThumbviewTopRightAction);
+	m_thumb_mode->addAction(m_ui.viewThumbviewBottomLeftAction);
+	m_thumb_mode->addAction(m_ui.viewThumbviewBottomRightAction);
+	m_thumb_mode->addAction(m_ui.viewThumbviewNoneAction);
+
+	m_ui.viewThumbviewTopLeftAction->setData(qpwgraph_thumb::TopLeft);
+	m_ui.viewThumbviewTopRightAction->setData(qpwgraph_thumb::TopRight);
+	m_ui.viewThumbviewBottomLeftAction->setData(qpwgraph_thumb::BottomLeft);
+	m_ui.viewThumbviewBottomRightAction->setData(qpwgraph_thumb::BottomRight);
+	m_ui.viewThumbviewNoneAction->setData(qpwgraph_thumb::None);
+
+	QObject::connect(m_ui.viewThumbviewTopLeftAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewThumbviewAction()));
+	QObject::connect(m_ui.viewThumbviewTopRightAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewThumbviewAction()));
+	QObject::connect(m_ui.viewThumbviewBottomLeftAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewThumbviewAction()));
+	QObject::connect(m_ui.viewThumbviewBottomRightAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewThumbviewAction()));
+	QObject::connect(m_ui.viewThumbviewNoneAction,
+		SIGNAL(triggered(bool)),
+		SLOT(viewThumbviewAction()));
+
 	QObject::connect(m_ui.viewTextBesideIconsAction,
 		SIGNAL(triggered(bool)),
 		SLOT(viewTextBesideIcons(bool)));
@@ -447,6 +482,22 @@ qpwgraph_form::qpwgraph_form (
 		 SLOT(helpAlsaMidi(bool)));
 #endif
 
+#ifdef CONFIG_SYSTEM_TRAY
+	m_ui.helpMenu->insertAction(
+		m_ui.helpAboutAction, m_ui.helpSystemTrayQueryCloseAction);
+	QObject::connect(m_ui.helpSystemTrayQueryCloseAction,
+		SIGNAL(triggered(bool)),
+		SLOT(helpSystemTrayQueryClose(bool)));
+#endif
+
+	m_ui.helpMenu->insertAction(
+		m_ui.helpAboutAction, m_ui.helpPatchbayQueryQuitAction);
+	QObject::connect(m_ui.helpPatchbayQueryQuitAction,
+		SIGNAL(triggered(bool)),
+		SLOT(helpPatchbayQueryQuit(bool)));
+	m_ui.helpMenu->insertSeparator(
+		m_ui.helpAboutAction);
+
 	QObject::connect(m_ui.helpAboutAction,
 		SIGNAL(triggered(bool)),
 		SLOT(helpAbout()));
@@ -494,6 +545,11 @@ qpwgraph_form::qpwgraph_form (
 // Destructor.
 qpwgraph_form::~qpwgraph_form (void)
 {
+	if (m_thumb)
+		delete m_thumb;
+
+	delete m_thumb_mode;
+
 #ifdef CONFIG_SYSTEM_TRAY
 	if (m_systray)
 		delete m_systray;
@@ -706,24 +762,63 @@ void qpwgraph_form::patchbayAutoDisconnect ( bool on )
 void qpwgraph_form::viewMenubar ( bool on )
 {
 	m_ui.MenuBar->setVisible(on);
+
+	++m_thumb_update;
 }
 
 
 void qpwgraph_form::viewGraphToolbar ( bool on )
 {
 	m_ui.graphToolbar->setVisible(on);
+
+	++m_thumb_update;
 }
 
 
 void qpwgraph_form::viewPatchbayToolbar ( bool on )
 {
 	m_ui.patchbayToolbar->setVisible(on);
+
+	++m_thumb_update;
 }
 
 
 void qpwgraph_form::viewStatusbar ( bool on )
 {
 	m_ui.StatusBar->setVisible(on);
+
+	++m_thumb_update;
+}
+
+
+void qpwgraph_form::viewThumbviewAction (void)
+{
+	QAction *action = qobject_cast<QAction *> (sender());
+	if (action)
+		viewThumbview(action->data().toInt());
+}
+
+
+void qpwgraph_form::viewThumbview ( int thumbview )
+{
+	const qpwgraph_thumb::Position position
+		= qpwgraph_thumb::Position(thumbview);
+	if (position == qpwgraph_thumb::None) {
+		if (m_thumb) {
+			m_thumb->hide();
+			delete m_thumb;
+			m_thumb = nullptr;
+			m_thumb_update = 0;
+		}
+	} else {
+		if (m_thumb) {
+			m_thumb->setPosition(position);
+		} else {
+			m_thumb = new qpwgraph_thumb(m_ui.graphCanvas, position);
+			m_thumb->show();
+			++m_thumb_update;
+		}
+	}
 }
 
 
@@ -736,6 +831,8 @@ void qpwgraph_form::viewTextBesideIcons ( bool on )
 		m_ui.graphToolbar->setToolButtonStyle(Qt::ToolButtonIconOnly);
 		m_ui.patchbayToolbar->setToolButtonStyle(Qt::ToolButtonIconOnly);
 	}
+
+	++m_thumb_update;
 }
 
 
@@ -757,6 +854,8 @@ void qpwgraph_form::viewRefresh (void)
 	if (m_ui.graphCanvas->isRepelOverlappingNodes())
 		++m_repel_overlapping_nodes; // fake nodes added!
 
+	++m_thumb_update;
+
 	refresh();
 }
 
@@ -893,6 +992,18 @@ void qpwgraph_form::helpAlsaMidi ( bool on )
 }
 
 
+void qpwgraph_form::helpSystemTrayQueryClose ( bool on )
+{
+	m_config->setSystemTrayQueryClose(on);
+}
+
+
+void qpwgraph_form::helpPatchbayQueryQuit ( bool on )
+{
+	m_config->setPatchbayQueryQuit(on);
+}
+
+
 void qpwgraph_form::helpAbout (void)
 {
 	static const QString title     = PROJECT_NAME;
@@ -1118,6 +1229,15 @@ void qpwgraph_form::renamed ( qpwgraph_item *item, const QString& name )
 }
 
 
+// Graph view change slot.
+void qpwgraph_form::changed (void)
+{
+	++m_thumb_update;
+
+	stabilize();
+}
+
+
 // Graph section slots.
 void qpwgraph_form::pipewire_changed (void)
 {
@@ -1167,6 +1287,13 @@ void qpwgraph_form::refresh (void)
 		m_repel_overlapping_nodes = 0;
 		m_ui.graphCanvas->repelOverlappingNodesAll();
 		stabilize();
+		++nchanged;
+	}
+
+	if (m_thumb_update > 0 || nchanged > 0) {
+		m_thumb_update = 0;
+		if (m_thumb)
+			m_thumb->updateView();
 	}
 
 	QTimer::singleShot(300, this, SLOT(refresh()));
@@ -1387,10 +1514,31 @@ bool qpwgraph_form::patchbayQueryQuit (void)
 		= m_ui.graphCanvas->patchbay();
 	if (patchbay && patchbay->isActivated()) {
 		showNormal();
-		ret = (QMessageBox::warning(this, tr("Warning"),
-			tr("A patchbay is currently activated:\n\n\"%1\"\n\n"
-			"Are you sure you want to quit?").arg(patchbayFileName()),
-			QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok);
+		if (m_ui.helpPatchbayQueryQuitAction->isChecked()) {
+			const QString& title
+				= tr("Warning");
+			const QString& text
+				= tr("A patchbay is currently activated:\n\n\"%1\"\n\n"
+					 "Are you sure you want to quit?").arg(patchbayFileName());
+		#if 0// Old no dont-ask-again message-box...
+			ret = (QMessageBox::warning(this, title, text,
+				QMessageBox::Ok | QMessageBox::Cancel) == QMessageBox::Ok);
+		#else
+			QMessageBox mbox(this);
+			mbox.setIcon(QMessageBox::Warning);
+			mbox.setWindowTitle(title);
+			mbox.setText(text);
+			mbox.setStandardButtons(QMessageBox::Ok | QMessageBox::Cancel);
+			QCheckBox cbox(tr("Don't ask this again"));
+			cbox.setChecked(false);
+			cbox.blockSignals(true);
+			mbox.addButton(&cbox, QMessageBox::ActionRole);
+			ret = (mbox.exec() == QMessageBox::Ok);
+			if (ret && cbox.isChecked()) {
+				m_ui.helpPatchbayQueryQuitAction->setChecked(false);
+			}
+		#endif
+		}
 	}
 
 	return ret;
@@ -1428,6 +1576,8 @@ void qpwgraph_form::resizeEvent ( QResizeEvent *event )
 {
 	QMainWindow::resizeEvent(event);
 
+	++m_thumb_update;
+
 	stabilize();
 }
 
@@ -1458,17 +1608,34 @@ void qpwgraph_form::closeEvent ( QCloseEvent *event )
 {
 #ifdef CONFIG_SYSTEM_TRAY
 	if (m_systray) {
-		if (!m_systray_closed) {
+		if (!m_systray_closed
+			&& m_ui.helpSystemTrayQueryCloseAction->isChecked()) {
 			const QString& title
 				= tr("Information");
 			const QString& text
 				= tr("The program will keep running in the system tray.\n\n"
 					"To terminate the program, please choose \"Quit\"\n"
-					"in the main menu or the context menu of the system tray icon.");
+					"in the context menu of the system tray icon.");
+		#if 0//--Old no dont-ask-again message-box...
 			if (QSystemTrayIcon::supportsMessages())
 				m_systray->showMessage(title, text, QSystemTrayIcon::Information);
 			else
 				QMessageBox::information(this, title, text);
+		#else
+			QMessageBox mbox(this);
+			mbox.setIcon(QMessageBox::Information);
+			mbox.setWindowTitle(title);
+			mbox.setText(text);
+			mbox.setStandardButtons(QMessageBox::Ok);
+			QCheckBox cbox(tr("Don't show this message again"));
+			cbox.setChecked(false);
+			cbox.blockSignals(true);
+			mbox.addButton(&cbox, QMessageBox::ActionRole);
+			mbox.exec();
+			if (cbox.isChecked()) {
+				m_ui.helpSystemTrayQueryCloseAction->setChecked(false);
+			}
+		#endif
 		}
 		m_systray_closed = true;
 		hide();
@@ -1617,6 +1784,27 @@ void qpwgraph_form::restoreState (void)
 	m_ui.viewPatchbayToolbarAction->setChecked(m_config->isPatchbayToolbar());
 	m_ui.viewStatusbarAction->setChecked(m_config->isStatusbar());
 
+	const qpwgraph_thumb::Position position
+		= qpwgraph_thumb::Position(m_config->thumbview());
+	switch (position) {
+	case qpwgraph_thumb::TopLeft:
+		m_ui.viewThumbviewTopLeftAction->setChecked(true);
+		break;
+	case qpwgraph_thumb::TopRight:
+		m_ui.viewThumbviewTopRightAction->setChecked(true);
+		break;
+	case qpwgraph_thumb::BottomLeft:
+		m_ui.viewThumbviewBottomLeftAction->setChecked(true);
+		break;
+	case qpwgraph_thumb::BottomRight:
+		m_ui.viewThumbviewBottomRightAction->setChecked(true);
+		break;
+	case qpwgraph_thumb::None:
+	default:
+		m_ui.viewThumbviewNoneAction->setChecked(true);
+		break;
+	}
+
 	m_ui.viewTextBesideIconsAction->setChecked(m_config->isTextBesideIcons());
 	m_ui.viewZoomRangeAction->setChecked(m_config->isZoomRange());
 	m_ui.viewRepelOverlappingNodesAction->setChecked(m_config->isRepelOverlappingNodes());
@@ -1656,6 +1844,8 @@ void qpwgraph_form::restoreState (void)
 	viewPatchbayToolbar(m_config->isPatchbayToolbar());
 	viewStatusbar(m_config->isStatusbar());
 
+	viewThumbview(m_config->thumbview());
+
 	viewTextBesideIcons(m_config->isTextBesideIcons());
 	viewZoomRange(m_config->isZoomRange());
 	viewRepelOverlappingNodes(m_config->isRepelOverlappingNodes());
@@ -1667,11 +1857,20 @@ void qpwgraph_form::restoreState (void)
 	m_patchbay_dir = m_config->patchbayDir();
 	m_patchbay_path = m_config->patchbayPath();
 
+	const bool is_patchbay_queryquit
+		= m_config->isPatchbayQueryQuit();
+	m_ui.helpPatchbayQueryQuitAction->setChecked(is_patchbay_queryquit);
+	helpPatchbayQueryQuit(is_patchbay_queryquit);
+
 #ifdef CONFIG_SYSTEM_TRAY
 	const bool is_systray_enabled
 		= m_config->isSystemTrayEnabled();
 	m_ui.helpSystemTrayAction->setChecked(is_systray_enabled);
 	helpSystemTray(is_systray_enabled);
+	const bool is_systray_queryclose
+		= m_config->isSystemTrayQueryClose();
+	m_ui.helpSystemTrayQueryCloseAction->setChecked(is_systray_queryclose);
+	helpSystemTrayQueryClose(is_systray_queryclose);
 #endif
 
 #ifdef CONFIG_ALSA_MIDI
@@ -1688,6 +1887,8 @@ void qpwgraph_form::saveState (void)
 {
 	m_ui.graphCanvas->saveState();
 
+	m_config->setThumbview(m_thumb ? m_thumb->position() : qpwgraph_thumb::None);
+
 	m_config->setTextBesideIcons(m_ui.viewTextBesideIconsAction->isChecked());
 	m_config->setZoomRange(m_ui.viewZoomRangeAction->isChecked());
 	m_config->setSortType(int(qpwgraph_port::sortType()));
@@ -1706,9 +1907,11 @@ void qpwgraph_form::saveState (void)
 	m_config->setPatchbayActivated(m_ui.patchbayActivatedAction->isChecked());
 	m_config->setPatchbayPath(m_patchbay_path);
 	m_config->setPatchbayDir(m_patchbay_dir);
+	m_config->setPatchbayQueryQuit(m_ui.helpPatchbayQueryQuitAction->isChecked());
 
 #ifdef CONFIG_SYSTEM_TRAY
 	m_config->setSystemTrayEnabled(m_ui.helpSystemTrayAction->isChecked());
+	m_config->setSystemTrayQueryClose(m_ui.helpSystemTrayQueryCloseAction->isChecked());
 #endif
 
 #ifdef CONFIG_ALSA_MIDI
