@@ -1,7 +1,7 @@
 // qpwgraph_port.cpp
 //
 /****************************************************************************
-   Copyright (C) 2021-2023, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2021-2024, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -176,12 +176,12 @@ void qpwgraph_port::setPortTitle ( const QString& title )
 	static const int MAX_TITLE_LENGTH = 29;
 	static const QString ellipsis(3, '.');
 
-	QString text = m_title;
+	QString text = m_title.simplified();
 	const int nlength = text.indexOf(':');
 	if (nlength >= 0)
 		text.remove(0, nlength + 1);
 	if (text.length() >= MAX_TITLE_LENGTH + ellipsis.length())
-		text = ellipsis + text.right(MAX_TITLE_LENGTH);
+		text = ellipsis + text.right(MAX_TITLE_LENGTH).trimmed();
 
 	m_text->setPlainText(text);
 
